# coding: utf-8

"""
    HRA-API

    This API provides programmatic access to data registered to the Human Reference Atlas (HRA). See the [HuBMAP HRA Portal](https://humanatlas.io/) for details. 

    The version of the OpenAPI document: 0.11.0
    Contact: help@hubmapconsortium.org
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from hra_api_client.models.tissue_dataset import TissueDataset
from hra_api_client.models.tissue_donor import TissueDonor
from hra_api_client.models.tissue_section import TissueSection
from typing import Optional, Set
from typing_extensions import Self

class TissueBlock(BaseModel):
    """
    Information about a collection of tissue sections.
    """ # noqa: E501
    sample_type: StrictStr = Field(alias="sampleType")
    spatial_entity_id: Optional[StrictStr] = Field(default=None, description="The associated spatial entity (rui_location) for the block.", alias="spatialEntityId")
    donor: Optional[TissueDonor] = None
    section_count: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Number of sections contained in the block.", alias="sectionCount")
    section_size: Optional[Union[Annotated[float, Field(strict=True, ge=0)], Annotated[int, Field(strict=True, ge=0)]]] = Field(default=None, description="Size of each section in the block.", alias="sectionSize")
    section_units: Optional[StrictStr] = Field(default='millimeter', description="Length unit `sectionSize` is in (generally, millimeters)", alias="sectionUnits")
    sections: Optional[List[TissueSection]] = Field(default=None, description="All sections in the block.")
    label: StrictStr
    description: Optional[StrictStr] = None
    link: StrictStr
    datasets: List[TissueDataset]
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["label", "description", "link", "sampleType", "datasets"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TissueBlock from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in datasets (list)
        _items = []
        if self.datasets:
            for _item_datasets in self.datasets:
                if _item_datasets:
                    _items.append(_item_datasets.to_dict())
            _dict['datasets'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TissueBlock from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "label": obj.get("label"),
            "description": obj.get("description"),
            "link": obj.get("link"),
            "sampleType": obj.get("sampleType"),
            "datasets": [TissueDataset.from_dict(_item) for _item in obj["datasets"]] if obj.get("datasets") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


